import kotlinx.kover.api.KoverTaskExtension
import kotlinx.kover.api.VerificationValueType
import org.jetbrains.kotlin.gradle.tasks.KotlinCompile

plugins {
    id("io.spring.dependency-management") version "1.0.11.RELEASE"
    id("org.springframework.boot") version "2.6.1"
    kotlin("jvm") version "1.6.0"
    kotlin("plugin.spring") version "1.6.0"
    idea
    id("org.jetbrains.kotlinx.kover") version "0.4.4"
    id("com.softeq.gradle.itest") version "1.0.4"
}

group = "org.example"
version = "1.0-SNAPSHOT"

repositories {
    mavenCentral()
}

configurations {
    itestImplementation.get().extendsFrom(testImplementation.get())
}

dependencies {
    implementation("org.springframework.boot:spring-boot-starter-validation")
    implementation("org.springframework.boot:spring-boot-starter-web:${property("springWebVersion")}")

    testImplementation("org.assertj:assertj-core:${property("assertJVersion")}")
    testImplementation("org.springframework.boot:spring-boot-starter-test:${property("springWebVersion")}")
    testImplementation("org.testcontainers:junit-jupiter:${property("testcontainersVersion")}")
}

idea {
    module {
        inheritOutputDirs = false

        testSourceDirs.plusAssign(sourceSets["itest"].allSource.srcDirs)
        testResourceDirs.plusAssign(project.sourceSets["itest"].resources.srcDirs)
    }
}

fun setupEnv(task: Test) {
    val env = System.getenv("SMOKE_TEST_ENV") ?: System.getenv("ENV") ?: "test"
    task.environment("SMOKE_TEST_ENV", env)
}

tasks {
    withType<KotlinCompile> {
        kotlinOptions {
            freeCompilerArgs = listOf("-Xjsr305=strict")
            jvmTarget = JavaVersion.VERSION_11.toString()
        }
    }

    withType<Test> {
        useJUnitPlatform {
            excludeTags("smoke")
        }

        extensions.configure(KoverTaskExtension::class) {
            includes = listOf("org\\.example\\..+")
            excludes = listOf(
                ".+Test",
            )
        }
    }

    register<Test>("smokeTest") {
        group = "verification"
        setupEnv(this)
        val activeProfile = System.getProperty("spring.profiles.active")
        systemProperty("spring.profiles.active", activeProfile)

        useJUnitPlatform {
            includeTags("smoke")
        }

        extensions.configure(KoverTaskExtension::class) {
            isEnabled = false
        }
    }

    koverVerify {
        rule {
            name = "The project has upper limit on lines covered"
            bound {
                minValue = 85
                valueType = VerificationValueType.COVERED_LINES_PERCENTAGE
            }
        }
    }
}